package com.isu.gaswellwatch.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.isu.gaswellwatch.constants.UserConstant;
import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.dao.MenuDao;
import com.isu.gaswellwatch.dto.MenuDTO;
import com.isu.gaswellwatch.dto.MenuEditDTO;
import com.isu.gaswellwatch.dto.MenuSortDTO;
import com.isu.gaswellwatch.entity.Menu;
import com.isu.gaswellwatch.entity.RoleMenu;
import com.isu.gaswellwatch.service.MenuService;
import com.isu.gaswellwatch.service.RoleMenuService;
import com.isu.gaswellwatch.utils.ConverterUtil;
import com.isu.gaswellwatch.vo.MenuListVO;
import com.isu.gaswellwatch.vo.MenuTreeVO;
import com.isu.gaswellwatch.vo.MenuVO;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.util.*;
import java.util.stream.Collectors;

@Service("menuService")
@Transactional(rollbackFor = Exception.class)
public class MenuServiceImpl extends ServiceImpl<MenuDao, Menu> implements MenuService {

    @Resource
    private MenuDao menuDao;
    @Resource
    private SnowflakeConfig snowflakeConfig;
    @Resource
    private RoleMenuService roleMenuService;

    @Override
    public List<MenuTreeVO> getMenuTreeByRoleId(Long roleId){
        if(roleId != null) return getMenuTreeByRoleIds(Collections.singletonList(roleId));
        return new ArrayList<>();
    }

    @Override
    public List<MenuListVO> getMenuListByRoleId(Long roleId){
        List<MenuVO> menuVOS = menuDao.selectRoleMenu(Collections.singletonList(roleId));
        if(CollectionUtil.isEmpty(menuVOS)) return new ArrayList<>();
        Map<Long, List<MenuVO>> map = menuVOS.stream().collect(Collectors.groupingBy(MenuVO::getId));
        return  map.entrySet().stream().map(entry -> {
            Long id = entry.getKey();
            List<MenuVO> objects = entry.getValue();
            List<String> identifier = objects.stream().map(MenuVO::getIdentifier).filter(StringUtils::isNotBlank).toList();
            return new MenuListVO(id, objects.getFirst().getParent(), objects.getFirst().getName(), objects.getFirst().getCode(),
                    objects.getFirst().getIcon(), objects.getFirst().getSort(), objects.getFirst().getExtra(), identifier);
        }).toList();
    }


    @Override
    public List<MenuTreeVO> getMenuTree(){
        List<MenuTreeVO> convert = ConverterUtil.convert(list(), MenuTreeVO.class);
        if(convert == null) return new ArrayList<>();
        return buildTree(convert, UserConstant.MENU_ROOT_ID);
    }

    @Override
    public void add(MenuDTO menuDTO){
        Menu convert = ConverterUtil.convert(menuDTO, Menu.class);
        long menuId = snowflakeConfig.snowflakeId();
        convert.setId(menuId);
        save(convert);
        //超管
        if(CollectionUtil.isEmpty(menuDTO.getIdentifier())){
            roleMenuService.save(RoleMenu.builder().id(snowflakeConfig.snowflakeId()).roleId(UserConstant.SUPER_ADMIN_ID).menuId(menuId).build());
        }else{
            List<RoleMenu> list = new ArrayList<>();
            menuDTO.getIdentifier().forEach(s->{
                list.add(RoleMenu.builder().id(snowflakeConfig.snowflakeId()).roleId(UserConstant.SUPER_ADMIN_ID).menuId(menuId).identifier(s).build());
            });
            roleMenuService.saveBatch(list);
        }
    }

    @Override
    public void edit(MenuEditDTO menuEditDTO){
        update(new UpdateWrapper<Menu>().lambda().eq(Menu::getId,menuEditDTO.getId()).set(Menu::getName,menuEditDTO.getName())
                .set(Menu::getExtra,menuEditDTO.getExtra()));
    }

    @Override
    public void delete(List<Long> ids){
        //删除菜单
        removeBatchByIds(ids);
        //删除菜单与角色关联
        roleMenuService.remove(new LambdaQueryWrapper<RoleMenu>().in(RoleMenu::getMenuId,ids));
    }

    @Override
    public void editSort(List<MenuSortDTO> menuSortDTOList){
        //更新菜单信息
        if(CollectionUtil.isEmpty(menuSortDTOList)) return;
        Map<Long, MenuSortDTO> map = menuSortDTOList.stream().collect(Collectors.toMap(MenuSortDTO::getId, menuSortDTO -> menuSortDTO, (v1, v2) -> v2));
        List<Long> list = menuSortDTOList.stream().map(MenuSortDTO::getId).toList();
        List<Menu> menuList = list(new LambdaQueryWrapper<Menu>().in(Menu::getId, list));
        if(!CollectionUtil.isEmpty(menuList)){
            menuList.forEach(menu -> {
               menu.setSort(map.get(menu.getId()).getSort());
               if(map.get(menu.getId()).getParent() != null){
                   menu.setParent(map.get(menu.getId()).getParent());
               }
            });
        }
        updateBatchById(menuList);
        //将移动后未勾选的父菜单也勾选
        List<RoleMenu> res = new ArrayList<>();
        Long menuId = null;
        List<Long> ancestors = null;
        for(MenuSortDTO menuSortDTO : menuSortDTOList){
            if(!CollectionUtil.isEmpty(menuSortDTO.getAncestors())){
                menuId = menuSortDTO.getId();
                ancestors = menuSortDTO.getAncestors();
            }
        }
        if(menuId != null){
            List<RoleMenu> roleMenuList = roleMenuService.list(new LambdaQueryWrapper<RoleMenu>().eq(RoleMenu::getMenuId, menuId));
            if(!CollectionUtil.isEmpty(roleMenuList)){
                List<Long> roleIds = roleMenuList.stream().map(RoleMenu::getRoleId).toList();
                List<RoleMenu> roleMenus = roleMenuService.list(new LambdaQueryWrapper<RoleMenu>().in(RoleMenu::getRoleId, roleIds));
                //带有拖动菜单对应的每个角色的所有菜单
                Map<Long, List<Long>> collect = roleMenus.stream().collect(Collectors.groupingBy(RoleMenu::getRoleId, Collectors.mapping(RoleMenu::getMenuId, Collectors.toList())));
                for(Map.Entry<Long, List<Long>> entry : collect.entrySet()){
                    Set<Long> set = new HashSet<>(entry.getValue());
                    for(Long ancestor : ancestors){
                        if(!set.contains(ancestor)){
                            res.add(RoleMenu.builder().id(snowflakeConfig.snowflakeId()).roleId(entry.getKey()).menuId(ancestor).build());
                        }
                    }
                }
                roleMenuService.saveBatch(res);
            }
        }

    }

    public List<MenuTreeVO> getMenuTreeByRoleIds(List<Long> ids){
        List<MenuVO> menuVOS = menuDao.selectRoleMenu(ids);
        if(CollectionUtil.isEmpty(menuVOS)) return new ArrayList<>();
        Map<Long, List<MenuVO>> map = menuVOS.stream().collect(Collectors.groupingBy(MenuVO::getId));
        List<MenuTreeVO> collect = map.entrySet().stream().map(entry -> {
            Long id = entry.getKey();
            List<MenuVO> objects = entry.getValue();
            List<String> identifier = objects.stream().map(MenuVO::getIdentifier).filter(StringUtils::isNotBlank).toList();
            return new MenuTreeVO(id, objects.getFirst().getParent(), objects.getFirst().getName(), objects.getFirst().getCode(),
                    objects.getFirst().getIcon(), objects.getFirst().getSort(), objects.getFirst().getExtra(), identifier, null);
        }).toList();
        if(CollectionUtil.isEmpty(collect)) return new ArrayList<>();
        return buildTree(collect,UserConstant.MENU_ROOT_ID);
    }

    private List<MenuTreeVO> buildTree(List<MenuTreeVO> menuTreeVO,Long parentId){
        return menuTreeVO.stream()
                // 筛选父节点
                .filter(t -> t.getParent().equals(parentId))
                // 递归设置子节点
                .peek(item -> item.setChildren(buildTree(menuTreeVO,item.getId())))
                .sorted(Comparator.comparing(MenuTreeVO::getSort))
                .collect(Collectors.toList());
    }
}

